//
// test program for affine self-calibration
//

#include <iostream>
#include <iomanip>
#include <string>
#include <stdio.h>

#include "AffineSelfCalibration.h"
#include "utils.h"

using namespace Eigen;
using namespace std;
using namespace AffineSelfCalibration;

int main()
{
   MatrixXd     *pt0;
   char         filename[] = "data-all.dat";
   int          PtNum, CamNum;

   // read data
   if (!read_data(filename, &pt0, &CamNum, &PtNum))
   {
      std::cerr << "Data file open failed!" << std::endl;
      return 1;
   }

   std::cerr << "Input data read end!" << std::endl;

   double fl[CamNum];   // for paraperspective self-calibration
   for (int i = 0; i < CamNum; i++)
      fl[i] = Default_f0;

   // factorization
   MatrixXd     Motion, Shape;
   
   // symmetric affine camera
   symmetric_affine_selfcalibration(pt0, CamNum, PtNum, Motion, Shape);
   write_data("sym-reconst.dat", Shape);
   write_ply_data("sym-reconst.ply", Shape, 255, 255, 255);
   std::cerr << "Symmetric Affine Camera Selfcalibration done!" << std::endl;
   
   // paraperspective camera
   paraperspective_selfcalibration(pt0, fl, CamNum, PtNum, Motion, Shape);
   write_data("para-reconst.dat", Shape);
   write_ply_data("para-reconst.ply", Shape, 255, 255, 255);
   std::cerr << "Paraperspective Camera Selfcalibration done!" << std::endl;
   
   // weak perspective camera
   weak_perspective_selfcalibration(pt0, CamNum, PtNum, Motion, Shape);
   write_data("weak-reconst.dat", Shape);
   write_ply_data("weak-reconst.ply", Shape, 255, 255, 255);
   std::cerr << "Weak-perspective Camera Selfcalibration done!" << std::endl;
   
   // orthographic camera
   orthographic_selfcalibration(pt0, CamNum, PtNum, Motion, Shape);
   write_data("orth-reconst.dat", Shape);
   write_ply_data("orth-reconst.ply", Shape, 255, 255, 255);
   std::cerr << "Orthographic Camera Selfcalibration done!" << std::endl;
   
   return 0;
}
