//
// Data read functions
//

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

#include <Eigen/Dense>
#include "utils.h"

#include <stdio.h>
#include <stdlib.h>

using namespace Eigen;
using namespace std;

#define MAXBUF 8192

bool
read_data(const char filename[], MatrixXd **pt0, int *CamNum, int *PtNum)
{
   MatrixXd     *ptr;
   std::ifstream ifs(filename);

//   std::cerr << "input file = " << filename << std::endl;

   if (ifs.fail())
   {
      std::cerr << "Cannot open file:" << filename << std::endl;
      return false;
   }

   char buff[MAXBUF];

   // read # of cameras
   while (ifs.getline(buff, MAXBUF-1))
   {
//      std::cerr << "***" << buff << std::endl;
      if (buff[0] == '#')
         continue;
      else
         break;
   }
   if (!sscanf(buff, "%d", CamNum) == 1)
      return false;
   
   std::cerr << "# of cameras = " << *CamNum << std::endl;
   
   // read # of points
   while (ifs.getline(buff, MAXBUF-1))
   {
//      std::cerr << "***" << buff << std::endl;
      if (buff[0] == '#')
         continue;
      else
         break;
   }
   if (!sscanf(buff, "%d", PtNum) == 1)
      return false;

   std::cerr << "# of points = " << *PtNum << std::endl;
   
   // data allocation
   ptr = *pt0 = new MatrixXd[*PtNum];

   // skip comments
   while (ifs.getline(buff, MAXBUF-1))
   {
      if (buff[0] == '#')
         continue;
      else
         break;
   }

   // read data
   for (int p = 0; p < *PtNum; p++)
   {
      char *strp, *nstrp;
      
      ptr[p].resize(2,*CamNum);

      // copy buff
      strp = buff;
//      std::cerr << p << " *** " << strp << std::endl;
      
      for (int c = 0; c < *CamNum; c++)
      {
         double x0, y0;
         
         // x
         x0 = strtod(strp, &nstrp);
         if (*nstrp != '\0')
         {
            ptr[p].col(c)(0) = x0;
            strp = nstrp;
         }
         else
            return false;

         // y
         y0 = strtod(strp, &nstrp);
         if (*nstrp != '\0')
         {
            ptr[p].col(c)(1) = y0;
            strp = nstrp;
         }
         else
            return false;

//         std::cerr << "*** "
//                   << ptr[p].col(c)(0) << ","
//                   << ptr[p].col(c)(1) << std::endl;
      }
      // read next line
      ifs.getline(buff, MAXBUF-1);
   }

   return true;
}

bool
write_data(const char filename[], const MatrixXd& M)
{
   int  Num;
   std::ofstream ofs(filename);

   if (ofs.fail())
   {
      std::cerr << "Cannot open file:" << filename << std::endl;
      return false;
   }

   // output the number of data
   Num = M.cols();
   ofs << Num << std::endl;

   // set output format
   ofs << scientific << setprecision(15);

   // output data
   for (int i = 0; i < Num; i++)
      ofs << M(0,i) << " "
          << M(1,i) << " "
          << M(2,i) << std::endl;

   return true;
}

// output ply format
bool
write_ply_data(const char filename[], const MatrixXd& M, int r, int g, int b)
{
   int  Num;
   std::ofstream ofs(filename);

   if (ofs.fail())
   {
      std::cerr << "Cannot open file:" << filename << std::endl;
      return false;
   }

   Num = M.cols();

   // wirte headers
   ofs << "ply" << endl
       << "format ascii 1.0" << endl
       << "comment created by 3d_to_ply" << endl
       << "element vertex " << Num << endl
       << "property float32 x" << endl
       << "property float32 y" << endl
       << "property float32 z" << endl
       << "property uint8 red" << endl
       << "property uint8 green" << endl
       << "property uint8 blue" << endl
       << "end_header" << endl;

   // output data
   for (int i = 0; i < Num; i++)
   {
      char buff[256];

      sprintf(buff, "%lf %lf %lf %d %d %d",
	      M(0,i), M(1,i), M(2,i), r, g, b);
      ofs << buff << endl;
   }
   
   return true;
}
