//
// test program for perspective camera self-calibration
//

#include <iostream>
#include <iomanip>
#include <string>
#include <stdio.h>

#include "PerspectiveSelfCalibration.h"
#include "utils.h"

using namespace Eigen;
using namespace std;
using namespace PerspectiveSelfCalibration;

int main()
{
   MatrixXd     *pt0;

   // read data from single file
/* 
   char         filename[] = "data-all.dat";
   int          PtNum, CamNum;

   if (!read_data(filename, &pt0, &CamNum, &PtNum))
   {
      std::cerr << "Data file open failed!" << std::endl;
      return 1;
   }
*/
   // read data from multiple files
   char         filetemplate[] = "sample/point-%03d.dat";
   int          CamNum = 100;
   int          PtNum;
   
   if (!read_data_files(filetemplate, CamNum, &pt0, &PtNum))
   {
      std::cerr << "Data file open failed!" << std::endl;
      return 1;
   }

   std::cerr << "# of cameras = " << CamNum << std::endl;
   std::cerr << "# of points = " << PtNum << std::endl;
   std::cerr << "Input data read end!" << std::endl;

   // factorization
   Matrix34d    Proj[CamNum];
   Vector4d     X[PtNum];

   // primary method: original version
   original_primary_method(pt0, CamNum, PtNum, Proj, X, 0.01, 600.0, 10000);
   write_4d_data("prim_reconst_orig_4d.dat", X, PtNum);
   std::cerr << "original primary method of Perspective Camera Selfcalibration done!" << std::endl;

   // primary method: faster version
   faster_primary_method(pt0, CamNum, PtNum, Proj, X, 0.01, 600.0, 10000);
   write_4d_data("prim_reconst_faster_4d.dat", X, PtNum);
   std::cerr << "faster primary method of Perspective Camera Selfcalibration done!" << std::endl;
   
   // dual method
   original_dual_method(pt0, CamNum, PtNum, Proj, X, 0.01, 600.0, 10000);
   write_4d_data("dual_reconst_orig_4d.dat", X, PtNum);
   std::cerr << "original dual method of Perspective Camera Selfcalibration done!" << std::endl << std::endl;

   faster_dual_method(pt0, CamNum, PtNum, Proj, X, 0.01, 600.0, 10000);
   write_4d_data("dual_reconst_faster_4d.dat", X, PtNum);
   std::cerr << "faster dual method of Perspective Camera Selfcalibration done!" << std::endl << std::endl;
}
