//
// test function for Fundamental Matrix Computation
//

#include <iostream>
#include <iomanip>
#include <string>

#include "FMatrixComputation.h"
#include "utils.h"

using namespace Eigen;
using namespace std;

int main()
{
   double *x0, *y0, *x1, *y1;
   int Num1, Num2, Num;
   char filename1[] = "data1.dat";
   char filename2[] = "data2.dat";

   // read data
   if (!read_data(filename1, &x0, &y0, &Num1) ||
       !read_data(filename2, &x1, &y1, &Num2))
      return 1;

   if (Num1 != Num2)
      return 1;
   
   Num = Num1;

   // convert data
   Vector2d pos0[Num];
   Vector2d pos1[Num];
   for (int al = 0; al < Num; al++)
   {
      pos0[al] << x0[al], y0[al];
      pos1[al] << x1[al], y1[al];
   }

   // compute Fundamental Matrix
   Vector9d theta;
   FMatrixComputation::least_squares(pos0, pos1, Num, theta);
   cout << scientific << setprecision(15)
        << "least-squares:F = \n"
        << theta(0) << " " << theta(1) << " " << theta(2) << "\n"
        << theta(3) << " " << theta(4) << " " << theta(5) << "\n"
        << theta(6) << " " << theta(7) << " " << theta(8) << endl;

   FMatrixComputation::taubin(pos0, pos1, Num, theta);
   cout << scientific << setprecision(15)
        << "taubin:F = \n"
        << theta(0) << " " << theta(1) << " " << theta(2) << "\n"
        << theta(3) << " " << theta(4) << " " << theta(5) << "\n"
        << theta(6) << " " << theta(7) << " " << theta(8) << endl;

   FMatrixComputation::efns(pos0, pos1, Num, theta);
   cout << scientific << setprecision(15)
        << "enfs:F = \n"
        << theta(0) << " " << theta(1) << " " << theta(2) << "\n"
        << theta(3) << " " << theta(4) << " " << theta(5) << "\n"
        << theta(6) << " " << theta(7) << " " << theta(8) << endl;

   FMatrixComputation::ransac(pos0, pos1, Num, theta);
   cout << scientific << setprecision(15)
        << "ransac:F = \n"
        << theta(0) << " " << theta(1) << " " << theta(2) << "\n"
        << theta(3) << " " << theta(4) << " " << theta(5) << "\n"
        << theta(6) << " " << theta(7) << " " << theta(8) << endl;

   return 0;
}
