//
// Small functions for Fundamental Matrix Computation
//

#include <iostream>
#include <fstream>
#include <string>

#include "FMatrixComputation.h"
#include "utils.h"

using namespace Eigen;
using namespace std;

#define MAXBUF 256

bool
read_data(char filename[], double **x0, double **y0, int *Num)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file" << std::endl;
      return false;
   }

   char buff[MAXBUF];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%d", Num);
   if (n != 1) return false;

   *x0 = new double [*Num];
   *y0 = new double [*Num];

   for (int i = 0; i < *Num; i++)
   {
      if (ifs.eof())  return  false;
      ifs.getline(buff, MAXBUF-1);
      n = sscanf(buff, "%lf %lf", (*x0 + i), (*y0 + i));
      if (n == EOF)
      { // null line
         i--;
         continue;
      }
      else if (n != 2) return false;
   }

   return true;
}


bool
read_data(char filename[], double **x0, double **y0, double **x1, double **y1,
          int *Num)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file" << std::endl;
      return false;
   }

   char buff[MAXBUF];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%d", Num);
   if (n != 1) return false;

   *x0 = new double [*Num];
   *y0 = new double [*Num];
   *x1 = new double [*Num];
   *y1 = new double [*Num];

   for (int i = 0; i < *Num; i++)
   {
      if (ifs.eof())  return  false;
      ifs.getline(buff, MAXBUF-1);
      n = sscanf(buff, "%lf %lf %lf %lf",
                 (*x0 + i), (*y0 + i), (*x1 + i), (*y1 + i));
      if (n == EOF)
      { // null line
         i--;
         continue;
      }
      if (n != 4) return false;
   }

   return true;
}
