//
// test function for 3-D reconstruction from two views
//
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

#include "TwoViewReconstruction.h"
#include "utils.h"

// namespaces
using namespace std;
using namespace Eigen;

int
main(int argc, char* argv[])
{
   int Num;
   char filename[] = "sample.dat";
   char fmatname[] = "F.dat";
   double f0 = TwoViewReconstruction::Default_f0;

   double x0[MAXPOINTS], y0[MAXPOINTS], x1[MAXPOINTS], y1[MAXPOINTS];
   int PtNum;

   // read data
   if (!read_data(filename, x0, y0, x1, y1, &PtNum))
      return 1;

   // convert data
   Vector2d p0[PtNum], p1[PtNum];
   for (int al = 0; al < PtNum; al++)
   {
      p0[al] << x0[al], y0[al];
      p1[al] << x1[al], y1[al];
   }

   // read fundamental matrix
   Matrix3d F;
   if (!read_F_matrix(fmatname, F))
      return 1;

   // focal lengths computation
   double nfl0, nfl1;

   // focal length computation
   TwoViewReconstruction::focal_length_computation(F, &nfl0, &nfl1);

   cout << "fl0 = " << nfl0 << endl;
   cout << "fl1 = " << nfl1 << endl;

   // motion parameters
   Matrix3d R;
   Vector3d t;
   
   TwoViewReconstruction::motion_parameter_computation(F, nfl0, nfl1, p0, p1, PtNum, R, t);

   cout << "R = \n" << R << endl;
   cout << "t = " << t.transpose() << endl;
   
   // 3-D reconstruction
   Vector3d X[PtNum];
   TwoViewReconstruction::reconstruction(p0, p1, PtNum, F, R, t, nfl0, nfl1, X);

   // output data
   char outfile[] = "output.dat";
   write_3D_data(outfile, X, PtNum);

   // output ply
   char plyfile[] = "output.ply";
   write_ply_data(plyfile, X, PtNum, 255, 255, 255);
}
