//
// file input/output
//

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

#include "utils.h"

using namespace std;
using namespace Eigen;

#define MAXLEN 1024

// read 2-D data
bool
read_data(char fname[],
          double *x0, double *y0,
          double *x1, double *y1,
          int *Num)
{
   ifstream ifs(fname);

   if (ifs.fail())
   {
      cerr << "File '" << fname << "' does not exist!\n";
      return false;
   }

   int NNum;
   string str;

   int i = 0;
   while (getline(ifs, str) && i < MAXPOINTS)
   {
      double x00, y00, x10, y10;

      if (sscanf(str.data(), "(%lg,%lg)--(%lg,%lg)",
		 &x00, &y00, &x10, &y10) != 4)
	 continue;

      x0[i] = x00;
      y0[i] = y00;
      x1[i] = x10;
      y1[i] = y10;
      i++;
   }

   if (i >= MAXPOINTS)
   {
      cerr << "The data number is exceeded! (> "
	   << MAXPOINTS << ")\n";
      return false;
   }

   *Num = i;

   return true;
}

// read F matrix
bool
read_F_matrix(char Fname[], Matrix3d& F)
{
   ifstream ifs(Fname);

   if (ifs.fail())
   {
      cerr << "File '" << Fname << "' cannot open!\n";
      return false;
   }

   int NNum;
   string str;
   double f00, f01, f02, f10, f11, f12, f20, f21, f22;

   getline(ifs, str);
   sscanf(str.data(), "%lg %lg %lg", &f00, &f01, &f02);
   getline(ifs, str);
   sscanf(str.data(), "%lg %lg %lg", &f10, &f11, &f12);
   getline(ifs, str);
   sscanf(str.data(), "%lg %lg %lg", &f20, &f21, &f22);

   F << f00, f01, f02, f10, f11, f12, f20, f21, f22;

   return true;
}

// output 3-D
bool
write_3D_data(char fname[], Vector3d X[], int Num)
{
   ofstream ofs(fname);

   if (ofs.fail())
   {
      cerr << "File '" << fname << "' cannot create!\n";
      return false;
   }

   ofs << Num << endl;

   ofs << setprecision(15);
   ofs << setiosflags(ios::scientific);

   for (int i = 0; i < Num; i++)
   {
      ofs << X[i](0) << " "
	  << X[i](1) << " "
	  << X[i](2) << endl;
   }

   return true;
}

// 3Dデータをply形式で出力
bool
write_ply_data(char plyname[],
               Vector3d X[], int Num,
               int r, int g, int b)
{
   ofstream ofs(plyname);

   if (ofs.fail())
   {
      cerr << "File '" << plyname << "' cannot create!\n";
      return false;
   }

   // wirte headers
   ofs << "ply" << endl
       << "format ascii 1.0" << endl
       << "comment created by simtest3" << endl
       << "element vertex " << Num << endl
       << "property float32 x" << endl
       << "property float32 y" << endl
       << "property float32 z" << endl
       << "property uint8 red" << endl
       << "property uint8 green" << endl
       << "property uint8 blue" << endl
       << "end_header" << endl;

   for (int i = 0; i < Num; i++)
   {
      char buff[MAXLEN];

      sprintf(buff, "%lf %lf %lf %d %d %d",
	      X[i](0), X[i](1), X[i](2), r, g, b);
      ofs << buff << endl;
   }

   return true;
}
