//
// test function for homography estimation
//
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

#include "HomographyComputation.h"
#include "utils.h"

using namespace Eigen;
using namespace std;

int main(int args, char** argv) {
   // variables
   Vector2d *pos0, *pos1;
   int num, num_a;
   char filename1[] = "data1.dat";
   char filename2[] = "data2.dat";
//   char filename1[] = "data1_true.dat";
//   char filename2[] = "data2_true.dat";

   // read data
   if (!read_data(filename1, &pos0, &num) ||
       !read_data(filename2, &pos1, &num_a)) {
      return 1;
   }

    if (num != num_a) {
        return 1;
    }

    // compute Homography matrix
    Matrix3d matH, matH_back;

    // set precision for cout
    cout << scientific << setprecision(15);

    HomographyComputation::least_squares(pos0, pos1, num, matH);
    cout << "least-squares:H = \n"
         << matH << endl;

    HomographyComputation::iterative_reweight(pos0, pos1, num, matH);
    cout << "iterative-reweight:H = \n"
         << matH << endl;

    HomographyComputation::taubin(pos0, pos1, num, matH);
    cout << "taubin:H = \n"
         << matH << endl;

    HomographyComputation::renormalization(pos0, pos1, num, matH);
    cout << "renormalization:H = \n"
         << matH << endl;

    HomographyComputation::hyper_renormalization(pos0, pos1, num, matH);
    cout << "hyper-renormalization:H = \n"
         << matH << endl;

    HomographyComputation::hyper_least_squares(pos0, pos1, num, matH);
    cout << "hyper-least-squares:H = \n"
         << matH << endl;

    HomographyComputation::fns(pos0, pos1, num, matH);
    cout << "fns:H = \n"
         << matH << endl;

    matH_back = matH; // backup the original FNS solution
    HomographyComputation::geometric_distance_minimization(pos0, pos1, num, matH);
    cout << "geometric distance minimization:H = \n"
         << matH << endl;

    // backup from the original FNS solution
    matH = matH_back;
    HomographyComputation::hyperaccurate_correction(pos0, pos1, num, matH);
    cout << "hyper accurate correction:H = \n"
         << matH << endl;

    bool is_inlier[num];
    HomographyComputation::ransac(pos0, pos1, num, matH, is_inlier);
    cout << "ransac:H = \n"
         << matH << endl;

    int num_inlier = 0;
    for (int i = 0; i < num; ++i) {
        if (is_inlier[i]) {
            ++num_inlier;
        }
    }

    cout << "# of inlier : " << num_inlier << endl;

    return 0;
}
