//
// test function for planar triangulation
//
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

#include "PlanarTriangulation.h"
#include "utils.h"

int main(int argc, char** argv) {
    using namespace Eigen;

    // variables
    Vector2d *pos0, *pos1;
    Matrix3d matH;

    // read data
    int num1, num2, num;
    char infile1[] = "noise1.dat";
    char infile2[] = "noise2.dat";
    char outfile1[] = "corrected1.dat";
    char outfile2[] = "corrected2.dat";
    char hmatname[] = "H.dat";

    if (!read_Hmat(hmatname, matH) ||
        !read_data(infile1, &pos0, &num1) ||
        !read_data(infile2, &pos1, &num2)) {
        std::cerr << "Data file open failed!" << std::endl;
        return 1;
    }

    if (num1 != num2) {
        return 1;
    }

    num = num1;

    // 3-D reconstruction;
    Vector3d X[num];
    bool success = PlanarTriangulation::optimal_correction(pos0, pos1, num, matH);
    if (!success) {
        return 1;
    }
    std::cout << "Optimal correction succeded!" << std::endl;

    // write result
    write_data(outfile1, pos0, num);
    write_data(outfile2, pos1, num);

    return 0;
}
