//
// Data read functions
//

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

#include "PlanarTriangulation.h"
#include "utils.h"

using namespace Eigen;
using namespace std;

#define MAXBUF 256

bool
read_Hmat(char filename[], Matrix3d& F)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file: " << filename << std::endl;
      return false;
   }

   char buff[MAXBUF];
   double f[9];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%lf %lf %lf", f, f+1, f+2);
   if (n != 3) return false;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%lf %lf %lf", f+3, f+4, f+5);
   if (n != 3) return false;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%lf %lf %lf", f+6, f+7, f+8);
   if (n != 3) return false;

   F << f[0], f[1], f[2], f[3], f[4], f[5], f[6], f[7], f[8];

   return true;
}

bool
read_Hmat(char filename[], Vector9d& theta)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file: " << filename << std::endl;
      return false;
   }

   char buff[MAXBUF];
   double f[9];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%lf %lf %lf %lf %lf %lf %lf %lf %lf",
              f, f+1, f+2, f+3, f+4, f+5, f+6, f+7, f+8);
   if (n != 9) return false;

   theta << f[0], f[1], f[2], f[3], f[4], f[5], f[6], f[7], f[8];

   return true;
}

bool
read_data(char filename[], Vector2d **pos0, int *Num)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file:" << filename << std::endl;
      return false;
   }

   char buff[MAXBUF];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%d", Num);
   if (n != 1) return false;

   *pos0 = new Vector2d [*Num];

   for (int i = 0; i < *Num; i++)
   {
      double x0, y0;
      if (ifs.eof())  return false;
      ifs.getline(buff, MAXBUF-1);
      n = sscanf(buff, "%lf %lf", &x0, &y0);
      if (n == EOF)
      { // null line
         i--;
         continue;
      }
      else if (n != 2) return false;
      (*pos0)[i] << x0, y0;
   }

   return true;
}

bool
read_data(char filename[], double **x0, double **y0, int *Num)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file:" << filename << std::endl;
      return false;
   }

   char buff[MAXBUF];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%d", Num);
   if (n != 1) return false;

   *x0 = new double [*Num];
   *y0 = new double [*Num];

   for (int i = 0; i < *Num; i++)
   {
      if (ifs.eof())  return  false;
      ifs.getline(buff, MAXBUF-1);
      n = sscanf(buff, "%lf %lf", (*x0 + i), (*y0 + i));
      if (n == EOF)
      { // null line
         i--;
         continue;
      }
      else if (n != 2) return false;
   }

   return true;
}


bool
read_data(char filename[], double **x0, double **y0, double **x1, double **y1,
          int *Num)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file" << std::endl;
      return false;
   }

   char buff[MAXBUF];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%d", Num);
   if (n != 1) return false;

   *x0 = new double [*Num];
   *y0 = new double [*Num];
   *x1 = new double [*Num];
   *y1 = new double [*Num];

   for (int i = 0; i < *Num; i++)
   {
      if (ifs.eof())  return  false;
      ifs.getline(buff, MAXBUF-1);
      n = sscanf(buff, "%lf %lf %lf %lf",
                 (*x0 + i), (*y0 + i), (*x1 + i), (*y1 + i));
      if (n == EOF)
      { // null line
         i--;
         continue;
      }
      if (n != 4) return false;
   }

   return true;
}

bool
write_data(char filename[], Vector2d *pos0, int Num)
{
   std::ofstream ofs(filename);

   if (ofs.fail())
   {
      std::cerr << "Cannot open file:" << filename << std::endl;
      return false;
   }

   // output the number of data
   ofs << Num << std::endl;

   // set output format
   ofs << scientific << setprecision(15);

   // output data
   for (int i = 0; i < Num; i++)
      ofs << pos0[i](0) << " " << pos0[i](1) << std::endl;

   return true;
}
