//
// test function for planar reconstruction
//
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

#include "PlaneReconstruction.h"
#include "utils.h"

using namespace Eigen;
using namespace std;

int main(int argc, char** argv) {
    // variables
    int num, num_a;
    Matrix3d matH;
    Vector2d *pos0, *pos1;

    char filename1[] = "data1.dat";
    char filename2[] = "data2.dat";
    char hmatname[] = "H.dat";
    char out3dfile[] = "3d-data.dat";
    char out3dplyfile[] = "3d-data.ply";

    // read data
    if (!read_data(filename1, &pos0, &num) ||
        !read_data(filename2, &pos1, &num_a) ||
        !read_Hmat(hmatname, matH)) {
        return 1;
    }

    if (num != num_a) {
        return 1;
    }

    // reconstruction
    double f = PlaneReconstruction::Default_f0;

    Vector3d np, nm;
    Vector3d tp, tm;
    Matrix3d Rp, Rm;
    double dist;
    bool check[4], check2[4];

    // check determinant
    if (matH.determinant() < 0) {
        matH = -matH;
    }

    // 1st check
    cout << "Choose solution by parameters." << endl;
    PlaneReconstruction::plane_reconstruction(matH, f, f, np, nm,
                                                &dist, tp, tm, Rp, Rm);

    bool success1 = PlaneReconstruction::choose_solution_by_parameters(
        np, nm, dist, Rp, Rm, true, true, check);
    if (!success1) {
        cerr << "Multiple solutions have been chosen by parameters." << endl;
    }

    for (int i = 0; i < 4; ++i) {
        if (check[i]) {
            cout << "solution " << (i + 1) << " is chosen." << endl;
        }
    }

    // 2nd check
    cout << "Choose solution by points." << endl;
    bool success2 = PlaneReconstruction::choose_solution_by_points(
        pos0, pos1, num, tp, Rp, tm, Rm, f, f, check);
    if (!success2) {
        cerr << "Multiple solutions have been chosen by points." << endl;
    }

    for (int i = 0; i < 4; ++i) {
        if (check[i]) {
            cout << "solution " << (i + 1) << " is chosen." << endl;
        }
    }

    if (!success1 || !success2) {
        cerr << "Failed to choose solution." << endl;
        return 1;
    }

    Vector3d n;
    Vector3d t;
    Matrix3d R;

    // which solusion?
    if (check[0]) {
        // 1st solution
        // np, tp
        n = np;
        t = tp;
        R = Rp;
    } else if (check[1]) {
        // 2nd solution
        // nm, tm
        n = nm;
        t = tm;
        R = Rm;
    } else if (check[2]) {
        // 3rd solution
        // -np, -tp
        n = -np;
        t = -tp;
        R = Rp;
    } else if (check[3]) {
        // 4th solution
        // -nm, -tm
        n = -nm;
        t = -tm;
        R = Rm;
    } else {
        // illegal pattern
        return 1;
    }

    // solution
    cout << "Solution:" << endl;
    cout << "n = \n" << n << endl;
    cout << "t = \n" << t << endl;
    cout << "R = \n" << R << endl;
    cout << "d = " << dist << endl;

    // 3-D reconstruction
    cout << "3-D Reconstruction..." << endl;
    Vector3d X[num];
    PlaneReconstruction::check_solution(pos0, pos1, num, t, R, f, f, X);

    write_3D_data(out3dfile, X, num);
    write_ply_data(out3dplyfile, X, num, 200, 200, 200);

    cout << "Done." << endl;
}
