//
// Small functions for ellipse fitting
//

#include <iostream>
#include <fstream>
#include <string>

#include "EllipseAnalysis.h"
#include "utils.h"

using namespace Eigen;
using namespace std;

#define MAXBUF 256

bool
read_data(char filename[], double **x, double **y, int *Num)
{
   std::ifstream ifs(filename);

   if (ifs.fail())
   {
      std::cerr << "Cannot open file" << std::endl;
      return false;
   }

   char buff[MAXBUF];
   int n;

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%d", Num);
   if (n != 1) return false;

   *x = new double [*Num];
   *y = new double [*Num];

   for (int i = 0; i < *Num; i++)
   {
      if (ifs.eof()) return false;
      ifs.getline(buff, MAXBUF-1);
      n = sscanf(buff, "%lf %lf", (*x + i), (*y + i));
      if (n == EOF)
      { // null line
         i--;
         continue;
      }
      if (n != 2) return false;
   }

   return true;
}

bool
read_Q(char filename[], Matrix3d& Q)
{
   char buff[MAXBUF];
   std::ifstream ifs(filename);

   double a, b, c, d, e, f;
   int n;

   if (ifs.fail())
   {
      std::cerr << "Cannot open file" << std::endl;
      return false;
   }

   ifs.getline(buff, MAXBUF-1);
   n = sscanf(buff, "%lf %lf %lf %lf %lf %lf", &a, &b, &c, &d, &e, &f);
   if (n != 6) return false;

   Q << a, b, d, b, c, e, d, e, f;

   return true;
}
